#include "DS1820.h"

unsigned char DS1820_Start_Conv ()
{
	unsigned char cnt;

	cnt = One_Wire_Reset();

	if (cnt) return cnt;

	One_Wire_Write_Byte(One_Wire_Skip_ROM);

	One_Wire_Write_Byte(DS1820_CONVERT_T_CMD);

	return 0;
}

unsigned char DS1820_Get_Temp (uint8_t *temp_code, uint8_t *subzero, uint8_t *cel_frac_bits)
{
	unsigned char cnt;

	uint8_t inbuff[DS1820_STRATCHPAD_SIZE], i = 0;

	cnt = One_Wire_Reset();

	if (cnt) return cnt;

	One_Wire_Write_Byte(One_Wire_Skip_ROM);

	One_Wire_Write_Byte(DS1820_READ_STRATCHPAD_CMD);

	for (cnt = 0; cnt != DS1820_STRATCHPAD_SIZE; cnt++) inbuff[cnt] = One_Wire_Read_Byte();

	if (Crc8Dallas(DS1820_STRATCHPAD_SIZE, inbuff)) return 4;

	uint16_t meas;

	meas = inbuff[0];  // LSB
	meas |= ((uint16_t)inbuff[1])<<8; // MSB

	// check for negative
	if ( meas & 0x8000 )  {
		*subzero = 1;      // mark negative
		meas ^= 0xffff;  // convert to positive => (twos complement)++
		meas++;
	}
	else *subzero = 0;

	i = inbuff[DS18B20_CONF_REG];
	if ( (i & DS18B20_11_BIT) == DS18B20_11_BIT )
		meas &= ~(DS18B20_11_BIT_UNDF);
	else if ( (i & DS18B20_10_BIT) == DS18B20_10_BIT )
		meas &= ~(DS18B20_10_BIT_UNDF);
	else { // if ( (i & DS18B20_9_BIT) == DS18B20_9_BIT ) {
		meas &= ~(DS18B20_9_BIT_UNDF);
	}

	*temp_code  = (uint8_t)(meas >> 4);
	*cel_frac_bits = (uint8_t)(meas & 0x000F);

	return 0;
}

/* converts to decicelsius
   input is ouput from meas_to_cel
   returns absolute value of temperatur in decicelsius
	i.e.: sz=0, c=28, frac=15 returns 289 (=28.9C)
0	0	0
1	625	625	1
2	1250	250
3	1875	875	3
4	2500	500	4
5	3125	125
6	3750	750	6
7	4375	375
8	5000	0
9	5625	625	9
10	6250	250
11	6875	875	11
12	7500	500	12
13	8125	125
14	8750	750	14
15	9375	375	*/
uint16_t DS1820_temp_to_decicel(uint8_t subzero, uint8_t cel, uint8_t cel_frac_bits)
{
	uint16_t h;
	uint8_t  i;
	uint8_t need_rounding[] = { 1, 3, 4, 6, 9, 11, 12, 14 };

	h = cel_frac_bits*DS18X20_FRACCONV/1000;
	h += cel*10;
	if (!subzero) {
		for (i = 0; i<sizeof(need_rounding); i++) {
			if ( cel_frac_bits == need_rounding[i] ) {
				h++;
				break;
			}
		}
	}
	return h;
}
