/*  09/01/2021
 *  Author: Oxford Electronics
 *  DS3231 driver
 */

#include "init.h"
#include "DS3231.h"

void I2C_init()
{
	RCC->APB1ENR |= RCC_APB1ENR_I2C2EN;
  I2C_DeInit(I2C2);
	I2C_InitTypeDef I2C_InitStructure;
	
	I2C_StructInit(&I2C_InitStructure);
	I2C_InitStructure.I2C_ClockSpeed = 10000;
	I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
	I2C_Init(I2C2, &I2C_InitStructure);
	I2C_Cmd(I2C2, ENABLE);
	I2C_AcknowledgeConfig(I2C2, ENABLE);
}

void I2C_Write(uint8_t HW_address, uint8_t addr, uint8_t data)
{
	I2C_GenerateSTART(I2C2, ENABLE);
		while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_MODE_SELECT));
	I2C_Send7bitAddress(I2C2, HW_address, I2C_Direction_Transmitter);
		while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));
	I2C_SendData(I2C2, addr);
		while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
	I2C_SendData(I2C2, data);
		while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
	I2C_GenerateSTOP(I2C2, ENABLE);
		while(I2C_GetFlagStatus(I2C2, I2C_FLAG_BUSY));
}

uint8_t I2C_Read(uint8_t HW_address, uint8_t addr)
{
	uint8_t data;
		while(I2C_GetFlagStatus(I2C2, I2C_FLAG_BUSY));
	I2C_GenerateSTART(I2C2, ENABLE);
		while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_MODE_SELECT));
	I2C_Send7bitAddress(I2C2, HW_address, I2C_Direction_Transmitter);
		while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));
	I2C_SendData(I2C2, addr);
		while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
	I2C_GenerateSTART(I2C2, ENABLE);
		while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_MODE_SELECT));
	I2C_Send7bitAddress(I2C2, HW_address, I2C_Direction_Receiver);
		while(!I2C_CheckEvent(I2C2,I2C_EVENT_MASTER_BYTE_RECEIVED));
	data = I2C_ReceiveData(I2C2);
		while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_BYTE_RECEIVED));
	I2C_AcknowledgeConfig(I2C2, DISABLE);
	I2C_GenerateSTOP(I2C2, ENABLE);
		while(I2C_GetFlagStatus(I2C2, I2C_FLAG_BUSY));
	return data;
}


uint8_t bcd (uint8_t data)
{
	uint8_t bc;

	bc=((((data&(1<<6))|(data&(1<<5))|(data&(1<<4)))*0x0A)>>4)+((data&(1<<3))|(data&(1<<2))|(data&(1<<1))|(data&0x01));

	return bc;
}

uint8_t bin(uint8_t dec){

	char bcd;

	char n, dig, num, count;

	num = dec;
	count = 0;
	bcd = 0;
	for (n = 0; n < 4; n++) {
		dig = num%10;
		num = num/10;
		bcd = (dig<<count)|bcd;
		count += 4;
	}

	return bcd;
}


void ds3231_init(uint8_t t_meas, uint8_t out)
{
	I2C_Write(ds3231_addr, ds3231_control, 0); //1 hz setup out square
}

void ds3231_set_time(uint8_t hour,uint8_t min,uint8_t sec)
{
	I2C_Write(ds3231_addr, ds3231_sec, bin(sec));
	I2C_Write(ds3231_addr, ds3231_min, bin(min));
	I2C_Write(ds3231_addr, ds3231_hour, bin(hour));
}

void ds3231_get_time(uint8_t *hour, uint8_t *min, uint8_t *sec)
{
	*sec = (bcd(I2C_Read(ds3231_addr, ds3231_sec)));
	*min = (bcd(I2C_Read(ds3231_addr, ds3231_min)));
	*hour = (bcd(I2C_Read(ds3231_addr, ds3231_hour)));
}

void ds3231_set_date(uint8_t day, uint8_t month, uint8_t year)
{
	I2C_Write(ds3231_addr, ds3231_date, bin(day));
	I2C_Write(ds3231_addr, ds3231_month, bin(month));
	I2C_Write(ds3231_addr, ds3231_year, bin(year));
}

void ds3231_get_date(uint8_t *day, uint8_t *month, uint8_t *year)
{
	*day = (bcd(I2C_Read(ds3231_addr, ds3231_date)));
	*month = (bcd(I2C_Read(ds3231_addr, ds3231_month)));
	*year = (bcd(I2C_Read(ds3231_addr, ds3231_year)));
}
